      SUBROUTINE opt_prop(abs_crs, aer_dtau, aer_gg, aer_ssa,
     $     dens, dtauc,  ice_dtau, ice_gg, ice_ssa, intgr, nlyr,
     $     nstr, pmom, ray_crs, ssalb,  wc_dtau, wc_gg, wc_ssa, zd,
     $     dtair, dto3, 
     $     dtaer, omaer, gaer,
     $     dtcld, omcld, gcld,
     $     dtice, omice, gice,
     $     z)
*
*BOD
*DESCRIPTION
* Calculates optical depth, single scattering albedo and
* phase function from absorption and scattering cross sections
* corresponding gas and particulate matter concentrations for 
* one wavelength.
*USAGE
**FORTRAN
*      CALL opt_prop(abs_crs, aer_dtau, aer_gg, aer_ssa,
*     $     dens, dtauc,  ice_dtau, ice_gg, ice_ssa, intgr, nlyr,
*     $     nstr, pmom, ray_crs, ssalb,  wc_dtau, wc_gg, wc_ssa, zd)  
*     $     dtair, dto3, dtaer, omaer, gaer, dtcld, omcld, gcld,
*     $     dtice, omice, gice,
*     $     z)
**PERL
*INPUT
**abs_crs
*     real array(temper,trace gas)
*     The absorption cross sections as a function of temper
*     for each absorbing trace gas.
**aer_dtau, aer_gg, aer_ssa
*     real array(layer)
*     The aerosol optical depth, asymmetry factor and single scattering 
*     albedo for each layer.
**dens
*     real array(levels,species)
*     The density of ndens species at nlyr+1 levels. The species index
*     must corresponds to the following trace gases if available. Water
*     vapor, carbon dioxide and nitrogen dioxide are presently not in
*     use and may be set to zero.
***1   Air
***2   ozone (o3)
***3   oxygen (o2)
***4   water vapor (h2o)
***5   carbon dioxide (co2)
***6   nitrogen dioxide (no2)
**ice_dtau, ice_gg, ice_ssa
*     real array(layer)
*     The ice cloud optical depth, asymmetry factor and single scattering 
*     albedo for each layer.
**intgr
*     integer
*     Type of behaviour for integrating density over layer.
***1  Linear behaviour assumed.
***2  Exponential behaviour assumed.
**nlyr
*     integer
*     Number of layers in the model atmosphere.
**nstr
*     integer
*     Number of streams, see disort.
**ray_crs
*     real array
*     The Rayleigh cross section.
**wc_dtau, wc_gg, wc_ssa
*     real array(layer)
*     The water cloud optical depth, asymmetry factor and single scattering 
*     albedo for each layer.
**zd
*     real array(0:*)
*     The altitude in km for levels. 
*OUTPUT
**dtauc
*     real array(*)
*     The optical depth of each layer, see disort.
**pmom
*     real array(streams,layer)
*     The moments of the phase function for each stream and layer.
*     See disort.
**ssalb
*     real array(*)
*     The single scattering albedo of each layer, see disort.
*ERROR_MESSAGES
*ROUTINES_CALLED
*FILES
*     param.inc                Include file
*REFERENCES
*EOD
*     
      IMPLICIT LOGICAL (A-Z)    ! To nearly get strong typing
      INCLUDE 'param.inc'
*
      INTEGER  intgr, nlyr, nstr
*
      REAL abs_crs(0:maxcly, *), aer_dtau(*), aer_gg(*),
     $     aer_ssa(*), dens(0:nlyr,*), dtauc(*), 
     $     ice_dtau(*), ice_gg(*), ice_ssa(*), 
     $     pmom(0:maxcmu,*), ray_crs, ssalb(*), 
     $     wc_dtau(*), wc_gg(*), wc_ssa(*), 
     $     zd(0:*),
     $     dtair(*), 
     $     dto3(*), 
     $     dtaer(*), omaer(*), gaer(*), 
     $     dtcld(*), omcld(*), gcld(*),
     $     dtice(*), omice(*), gice(*),
     $     z(*)
*
* Internal variables
*
      INTEGER k, lc
      LOGICAL first
      REAL babs, bsca, bext,
     $     babsa, babsd, babsi, babso, bscaa, bscad, bscai, bscar, 
     $     cdento(maxcly), deltaz, epsiln, ga, gd, gi, sigo3avg, sprec,
     $     sumsca
      REAL r1mach
*
      DATA first /.TRUE./
      DATA epsiln / 1.E-5 /
      SAVE cdento, first, sprec
*
      IF ( first ) THEN
         sprec = r1mach(1)                          
*     
* Calculate column density of air (total density of the atmosphere)
*     
         DO lc = 1, nlyr
            deltaz = ( zd(lc-1)- zd(lc) )*1.E+5  ! The factor 1.E+5 
                                                 ! converts z from km to cm.
            IF (dens(lc-1,iair).GT.0. .AND. dens(lc,iair).GT.0.) THEN 
               IF ((ABS(1.0 - dens(lc,iair)/
     $              dens(lc-1,iair))).GT.epsiln) THEN
                  cdento(lc) = 1. /
     $                 (LOG(dens(lc,iair) / dens(lc-1,iair))) * 
     $                 (dens(lc,iair) - dens(lc-1,iair)) * deltaz 
               ELSE
                  cdento(lc) = 0.5* ABS((dens(lc-1,iair) +
     $                 dens(lc,iair))*deltaz )
               ENDIF
            ELSE
               cdento(lc) = 0.5* ABS((dens(lc-1,iair) +
     $              dens(lc,iair) )*deltaz )
            ENDIF
cakyp
*            write(*,*)"dens",lc, dens(lc,iair), cdento(lc)
cakyp
         ENDDO
         FIRST = .FALSE.
      ENDIF
*
      DO lc = 1, nlyr
         bscar = cdento(lc)*ray_crs
cakyp         
*         write(*,*)"bscar",lc,cdento(lc), ray_crs
cakyp         
         deltaz = ( zd(lc-1) - zd(lc) ) * 1.E+5 ! Convert from km to cm
         IF ( intgr .EQ. 0 ) THEN ! Linear integration
            babso = 
     $           + 0.5*deltaz*( dens(lc-1,io3)*abs_crs(lc-1,io3)
     $           + dens(lc,io3)*abs_crs(lc,io3) )
         ELSE IF ( intgr .EQ. 1 ) THEN ! Exponential integration
            sigo3avg = 0.5 * ( abs_crs(lc-1,io3) + abs_crs(io3,lc) )
            babso = deltaz * (
     $           (dens(lc,io3)-dens(lc-1,io3))/ALOG(dens(lc,io3)
     $           /dens(lc-1,io3))* sigo3avg)
         ENDIF
*         write(*,*)"bscar",lc,bscar,cdento(lc), ray_crs, 
*     $        dens(lc,io3), abs_crs(lc,io3), babso, deltaz, zd(lc)
*     
* Phase function
*
         bscaa  = aer_ssa(lc)*aer_dtau(lc)
         bscad  = wc_ssa(lc)*wc_dtau(lc)
         bscai  = ice_ssa(lc)*ice_dtau(lc)
         ga     = aer_gg(lc)
         gd     = wc_gg(lc)
         gi     = ice_gg(lc)
         sumsca =  bscar + bscad +  bscaa + bscai
         IF ( sumsca.LT.sprec ) sumsca = sprec
         pmom(0,lc) = 1.0
         DO k=1,nstr
            pmom(k,lc) = (bscad*gd**k + bscaa*ga**k
     $           + bscai*gi**k ) / sumsca  
            IF ( k.EQ.2 ) pmom(k,lc) = pmom(k,lc) +
     $           bscar/( sumsca*10.)
cakyp
*                 write(*,*)"pmom",k,nstr,lc, pmom(k,lc),
*     $           sumsca, bscad, bscaa, bscai, bscar,gd, ga, gi
cakyp
         ENDDO
*     
*     Absorption coefficient       
*
         babsa  = (1.-aer_ssa(lc))*aer_dtau(lc)
         babsd  = (1.-wc_ssa(lc))*wc_dtau(lc)
         babsi  = (1.-ice_ssa(lc))*ice_dtau(lc)
         babs = babsa + babso + babsd + babsi
         IF ( babs .GT. r1mach(2) ) THEN
            babs = r1mach(2)
         ENDIF
*
*     Scattering coefficient  
*     
         bsca = bscar + bscad + bscaa + bscai
         IF ( bsca .GT. r1mach(2) ) THEN
            bsca = r1mach(2)
         ENDIF
*     
*     Extinction coefficient   
*     
         bext = babs + bsca        
         IF ( bext .GT. r1mach(2) ) THEN
            bext = r1mach(2)
         ENDIF
*
         dtair (nlyr+1-lc) = bscar
         dto3  (nlyr+1-lc) = babso
         dtaer (nlyr+1-lc) = babsa + bscaa
         dtcld (nlyr+1-lc) = babsd + bscad
         dtice (nlyr+1-lc) = babsi + bscai
*     
         omaer (nlyr+1-lc) = aer_ssa (lc)
         omcld (nlyr+1-lc) = wc_ssa  (lc)
         omice (nlyr+1-lc) = ice_ssa (lc)
*
         gaer  (nlyr+1-lc) = aer_gg (lc)
         gcld  (nlyr+1-lc) = wc_gg  (lc)
         gice  (nlyr+1-lc) = ice_gg (lc)
*
*     Set optical depth and single scattering albedo
*     
         IF (bext.LT.sprec) bext = sprec
         dtauc(lc) = bext                          
         ssalb(lc) = bsca/bext           
         IF ( 1.0-ssalb(lc).LT.sprec ) ssalb(lc) = 1.0 - sprec
cakyp
*         WRITE(*,*)"opt_prop", lc, bscaa, babsa, ga, babso, bscar 
cakyp
*
      ENDDO
*
      DO lc = 0, nlyr
         z (nlyr+1-lc) = zd (lc) 
      ENDDO
*
         dtair (nlyr+1) = 0.
         dto3  (nlyr+1) = 0.
         dtaer (nlyr+1) = 0.
         dtcld (nlyr+1) = 0.
         dtice (nlyr+1) = 0.
*     
         omaer (nlyr+1) = 0.
         omcld (nlyr+1) = 0.
         omice (nlyr+1) = 0.
*
         gaer  (nlyr+1) = 0.
         gcld  (nlyr+1) = 0.
         gice  (nlyr+1) = 0.
*
*
      RETURN
      END
